@@ -1,4 +1,4 @@
-.coxfit <- function(response, offset) {
+.coxfit <- function(response, offset, strata) {
 
   n <- nrow(response)
   type <- attr(response, "type")
@@ -17,7 +17,15 @@
     dtimes <- time[status==1]
     Riskset <- outer(time, dtimes, ">=") & outer(start, dtimes, "<")
   }
-  whichd <- which(status ==1)
+  whichd <- which(status==1)
+  if (!is.null(strata)) {
+    dstrata <- strata[status==1]
+    Riskset <- Riskset & outer(strata, dstrata, "==")
+  } else {
+    strata <- factor(rep("baseline", nrow(response)))
+    dstrata <- strata[status==1]
+  }
+  
 
   # Finds local gradient and subject weights
   fit <- function(lp, leftout) {
@@ -25,8 +33,10 @@
     if (!missing(leftout)) {
       status <- status[!leftout]
       time <- time[!leftout]
+      strata <- strata[!leftout]
       dleftout <- leftout[whichd]
       dtimes <- dtimes[!dleftout]
+      dstrata <- dstrata[!dleftout]
       Riskset <- Riskset[!leftout, !dleftout]
       offset <- offset[!leftout]
     }
@@ -57,23 +67,42 @@
     Pij <- outer(ws, breslows) * Riskset
     W <- list(P = Pij, diagW = breslow * ws)        # construct: W = diag(diagW) - P %*% t(P)
 
-    # The fitted baseline
-    sdtimes <- sort(dtimes)
-    basecumhaz <- cumsum(breslows[sort.list(dtimes)])
-    uniquetimes <- c(TRUE, as.logical(sapply(seq_along(sdtimes)[-length(sdtimes)], function(i) sdtimes[i] != sdtimes[i+1])))
-    basesurv <- exp(-basecumhaz[uniquetimes])
-    if (max(dtimes) < max(time)) {
-      basetimes <- c(0, sdtimes[uniquetimes], max(time))
-      basesurv <- c(1, basesurv, basesurv[length(basesurv)])
-    } else {
-      basetimes <- c(0, sdtimes[uniquetimes])
-      basesurv <- c(1, basesurv)
+    # The fitted baseline(s)
+    baseline <- function() {
+      sortlistdtimes <- sort.list(dtimes)
+      baselines <- lapply(levels(strata), function(stratum) {
+        stratumdtimes <- sortlistdtimes[dstrata == stratum]
+        if (length(stratumdtimes) > 0) {
+          sdtimes <- dtimes[stratumdtimes]
+          basecumhaz <- cumsum(breslows[stratumdtimes])
+          uniquetimes <- c(TRUE, as.logical(sapply(seq_along(sdtimes)[-length(sdtimes)], function(i) sdtimes[i] != sdtimes[i+1])))
+          basesurv <- exp(-basecumhaz[uniquetimes])
+          if (max(dtimes) < max(time)) {
+            basetimes <- c(sdtimes[uniquetimes], max(time))
+            basesurv <- c(basesurv, basesurv[length(basesurv)])
+          } else {
+            basetimes <- c(sdtimes[uniquetimes])
+            basesurv <- c(basesurv)
+          }
+          if (min(time) > 0) {
+            basetimes <- c(0, basetimes)
+            basesurv <- c(1, basesurv)
+          }
+        } else {
+          basetimes <- 0
+          basesurv <- 1
+        }
+    
+        baseline <- new("breslow")
+        baseline@time <- basetimes
+        baseline@curves <- matrix(basesurv,1,byrow=TRUE)
+        baseline
+      })
+      groups <- seq_along(levels(strata))
+      names(groups) <- levels(strata)
+      .coxmerge(baselines, groups)
     }
 
-    baseline <- new("breslow")
-    baseline@time <- basetimes
-    baseline@curves <- matrix(basesurv,1,byrow=TRUE)
-
     return(list(residuals = residuals, loglik = loglik, W = W, lp = lp, lp0 = lp0, fitted = exp(lp), nuisance = list(baseline = baseline)))
   }
 
@@ -96,16 +125,24 @@
     return(sum(cvls[leftout]))
   }
 
-  prediction <- .coxpredict
+  # cross-validated predictions
+  prediction <- function(lp, nuisance, which) {
+    if (!is.null(offset)) lp <- lp + offset
+    out <- nuisance$baseline()
+    out@curves <- out@curves[strata[which],,drop=FALSE]
+    out@curves <- out@curves ^ matrix(exp(lp[which]), nrow(out@curves), ncol(out@curves))
+    out
+  }
 
   return(list(fit = fit, cvl = cvl, prediction = prediction))
 }
 
 
 # mapping from the linear predictor lp to an actual prediction
-.coxpredict <- function(lp, nuisance) {
+.coxpredict <- function(lp, nuisance, strata) {
   out <- nuisance$baseline
-  out@curves <- t(outer(drop(nuisance$baseline@curves), exp(lp), "^"))
+  out@curves <- out@curves[strata,,drop=FALSE]
+  out@curves <- out@curves ^ matrix(exp(lp), nrow(out@curves), ncol(out@curves))
   out
 }
 
@@ -113,7 +150,6 @@
 # merges predicted survival curves with different time points
 # input: a list of breslow objects
 .coxmerge <- function(predictions, groups) {
-             
   times <- sort(unique(unlist(lapply(predictions, time))))
   curves <- lapply(predictions, function(pred) {
     res <- matrix(NA, nrow(pred@curves), length(times))
